@@ -3,63 +3,51 @@ mod search;
 pub mod utils;
 
 use crate::ansi::CursorShape;
+use crate::context::renderable::{Cursor, RenderableContent, RenderableContentStrategy};
+use crate::context::ContextManager;
 use crate::crosswords::grid::row::Row;
-use crate::crosswords::pos::{Column, CursorState, Line, Pos};
+use crate::crosswords::pos::{Column, Line, Pos};
 use crate::crosswords::square::{Flags, Square};
-use crate::ime::Preedit;
 use crate::screen::hint::HintMatches;
-use crate::selection::SelectionRange;
 use navigation::ScreenNavigation;
 use rio_backend::config::colors::{
     term::{List, TermColors},
     AnsiColor, ColorArray, Colors, NamedColor,
 };
 use rio_backend::config::Config;
+use rio_backend::event::EventProxy;
 use rio_backend::sugarloaf::{
     Content, FragmentStyle, FragmentStyleDecoration, Graphic, Stretch, Style,
     SugarCursor, Sugarloaf, UnderlineInfo, UnderlineShape, Weight,
 };
 use std::collections::HashMap;
 use std::ops::RangeInclusive;
-use std::time::{Duration, Instant};
 
 use rustc_hash::FxHashMap;
 use unicode_width::UnicodeWidthChar;
 
-struct Cursor {
-    state: CursorState,
-    content: char,
-    content_ref: char,
-}
-
 pub struct Renderer {
-    #[allow(unused)]
-    pub option_as_alt: String,
-    is_ime_enabled: bool,
     is_vi_mode_enabled: bool,
     pub is_kitty_keyboard_enabled: bool,
-    pub last_typing: Option<Instant>,
     pub named_colors: Colors,
-    font_size: f32,
     pub colors: List,
     pub navigation: ScreenNavigation,
-    cursor: Cursor,
-    pub selection_range: Option<SelectionRange>,
     pub config_has_blinking_enabled: bool,
     pub config_blinking_interval: u64,
-    term_has_blinking_enabled: bool,
-    pub is_blinking: bool,
     ignore_selection_fg_color: bool,
+    #[allow(unused)]
+    pub option_as_alt: String,
+    #[allow(unused)]
+    pub macos_use_unified_titlebar: bool,
     // Dynamic background keep track of the original bg color and
     // the same r,g,b with the mutated alpha channel.
     pub dynamic_background: ([f32; 4], wgpu::Color, bool),
-    hyperlink_range: Option<SelectionRange>,
-    active_search: Option<String>,
     font_context: rio_backend::sugarloaf::font::FontLibrary,
     font_cache: FxHashMap<
         (char, rio_backend::sugarloaf::font_introspector::Attributes),
         (usize, f32),
     >,
+    active_search: Option<String>,
 }
 
 impl Renderer {
@@ -92,15 +80,12 @@ impl Renderer {
         }
 
         Renderer {
+            macos_use_unified_titlebar: config.window.macos_use_unified_titlebar,
             config_blinking_interval: config.cursor.blinking_interval.clamp(350, 1200),
             option_as_alt: config.option_as_alt.to_lowercase(),
             is_kitty_keyboard_enabled: config.keyboard.use_kitty_keyboard_protocol,
-            is_ime_enabled: false,
             is_vi_mode_enabled: false,
-            is_blinking: false,
-            last_typing: None,
             config_has_blinking_enabled: config.cursor.blinking,
-            term_has_blinking_enabled: false,
             ignore_selection_fg_color: config.ignore_selection_fg_color,
             colors,
             navigation: ScreenNavigation::new(
@@ -108,52 +93,19 @@ impl Renderer {
                 color_automation,
                 config.padding_y,
             ),
-            font_size: config.fonts.size,
-            selection_range: None,
-            hyperlink_range: None,
             named_colors,
             dynamic_background,
             active_search: None,
-            cursor: Cursor {
-                content: config.cursor.shape.into(),
-                content_ref: config.cursor.shape.into(),
-                state: CursorState::new(config.cursor.shape.into()),
-            },
             font_cache: FxHashMap::default(),
             font_context: font_context.clone(),
         }
     }
 
-    #[inline]
-    pub fn has_blinking_enabled(&self) -> bool {
-        self.config_has_blinking_enabled && self.term_has_blinking_enabled
-    }
-
-    #[inline]
-    pub fn get_cursor_state_from_ref(&self) -> CursorState {
-        CursorState::new(self.cursor.content_ref)
-    }
-
-    #[inline]
-    pub fn get_cursor_state(&self) -> CursorState {
-        self.cursor.state.clone()
-    }
-
     #[inline]
     pub fn set_active_search(&mut self, active_search: Option<String>) {
         self.active_search = active_search;
     }
 
-    #[inline]
-    pub fn set_hyperlink_range(&mut self, hyperlink_range: Option<SelectionRange>) {
-        self.hyperlink_range = hyperlink_range;
-    }
-
-    #[inline]
-    pub fn has_hyperlink_range(&self) -> bool {
-        self.hyperlink_range.is_some()
-    }
-
     #[inline]
     fn create_style(&mut self, square: &Square) -> (FragmentStyle, char) {
         let flags = square.flags;
@@ -264,15 +216,22 @@ impl Renderer {
     }
 
     #[inline]
+    #[allow(clippy::too_many_arguments)]
     fn create_line(
         &mut self,
-        content_builder: &mut Content,
+        builder: &mut Content,
         row: &Row<Square>,
         has_cursor: bool,
+        line_opt: Option<usize>,
         line: Line,
+        renderable_content: &RenderableContent,
         search_hints: &mut Option<HintMatches>,
         focused_match: &Option<RangeInclusive<Pos>>,
+        is_active: bool,
     ) {
+        let cursor = &renderable_content.cursor;
+        let hyperlink_range = renderable_content.hyperlink_range;
+        let selection_range = renderable_content.selection_range;
         let columns: usize = row.len();
         let mut content = String::default();
         let mut last_char_was_space = false;
@@ -286,16 +245,15 @@ impl Renderer {
             }
 
             let (mut style, square_content) =
-                if has_cursor && column == self.cursor.state.pos.col {
-                    self.create_cursor_style(square)
+                if has_cursor && column == cursor.state.pos.col {
+                    self.create_cursor_style(square, cursor, is_active)
                 } else {
                     self.create_style(square)
                 };
 
-            if self.hyperlink_range.is_some()
+            if hyperlink_range.is_some()
                 && square.hyperlink().is_some()
-                && self
-                    .hyperlink_range
+                && hyperlink_range
                     .unwrap()
                     .contains(Pos::new(line, Column(column)))
             {
@@ -306,9 +264,8 @@ impl Renderer {
                         is_doubled: false,
                         shape: UnderlineShape::Regular,
                     }));
-            } else if self.selection_range.is_some()
-                && self
-                    .selection_range
+            } else if selection_range.is_some()
+                && selection_range
                     .unwrap()
                     .contains(Pos::new(line, Column(column)))
             {
@@ -395,7 +352,11 @@ impl Renderer {
             if square_content == ' ' {
                 if !last_char_was_space {
                     if !content.is_empty() {
-                        content_builder.add_text(&content, last_style);
+                        if let Some(line) = line_opt {
+                            builder.add_text_on_line(line, &content, last_style);
+                        } else {
+                            builder.add_text(&content, last_style);
+                        }
                         content.clear();
                     }
 
@@ -404,7 +365,11 @@ impl Renderer {
                 }
             } else {
                 if last_char_was_space && !content.is_empty() {
-                    content_builder.add_text(&content, last_style);
+                    if let Some(line) = line_opt {
+                        builder.add_text_on_line(line, &content, last_style);
+                    } else {
+                        builder.add_text(&content, last_style);
+                    }
                     content.clear();
                 }
 
@@ -413,7 +378,11 @@ impl Renderer {
 
             if last_style != style {
                 if !content.is_empty() {
-                    content_builder.add_text(&content, last_style);
+                    if let Some(line) = line_opt {
+                        builder.add_text_on_line(line, &content, last_style);
+                    } else {
+                        builder.add_text(&content, last_style);
+                    }
                     content.clear();
                 }
 
@@ -425,14 +394,22 @@ impl Renderer {
             // Render last column and break row
             if column == (columns - 1) {
                 if !content.is_empty() {
-                    content_builder.add_text(&content, last_style);
+                    if let Some(line) = line_opt {
+                        builder.add_text_on_line(line, &content, last_style);
+                    } else {
+                        builder.add_text(&content, last_style);
+                    }
                 }
 
                 break;
             }
         }
 
-        content_builder.new_line();
+        if let Some(line) = line_opt {
+            builder.build_line(line);
+        } else {
+            builder.new_line();
+        }
     }
 
     #[inline]
@@ -563,22 +540,13 @@ impl Renderer {
         }
     }
 
-    // #[inline]
-    // #[allow(dead_code)]
-    // fn create_graphic_sugar(&self, square: &Square) -> Sugar {
-    //     let media = &square.graphics().unwrap()[0].texture;
-    //     Sugar {
-    //         media: Some(SugarGraphic {
-    //             id: media.id,
-    //             width: media.width,
-    //             height: media.height,
-    //         }),
-    //         ..Sugar::default()
-    //     }
-    // }
-
     #[inline]
-    fn create_cursor_style(&self, square: &Square) -> (FragmentStyle, char) {
+    fn create_cursor_style(
+        &self,
+        square: &Square,
+        cursor: &Cursor,
+        is_active: bool,
+    ) -> (FragmentStyle, char) {
         let font_attrs = match (
             square.flags.contains(Flags::ITALIC),
             square.flags.contains(Flags::BOLD_ITALIC),
@@ -593,8 +561,8 @@ impl Renderer {
         let mut color = self.compute_color(&square.fg, square.flags);
         let mut background_color = self.compute_bg_color(square);
         // If IME is enabled we get the current content to cursor
-        let content = if self.is_ime_enabled {
-            self.cursor.content
+        let content = if cursor.is_ime_enabled {
+            cursor.content
         } else {
             square.c
         };
@@ -603,15 +571,36 @@ impl Renderer {
             std::mem::swap(&mut background_color, &mut color);
         }
 
-        // If IME is enabled or is a block cursor, put background color
+        let has_dynamic_background = self.dynamic_background.2
+            && background_color[0] == self.dynamic_background.0[0]
+            && background_color[1] == self.dynamic_background.0[1]
+            && background_color[2] == self.dynamic_background.0[2];
+        let background_color = if has_dynamic_background
+            && (cursor.state.content != CursorShape::Block && is_active)
+        {
+            None
+        } else {
+            Some(background_color)
+        };
+
+        // If IME is or cursor is block enabled, put background color
         // when cursor is over the character
-        if self.is_ime_enabled || self.cursor.state.content == CursorShape::Block {
-            color = self.named_colors.background.0;
+        match (
+            cursor.is_ime_enabled,
+            (cursor.state.content == CursorShape::Block || !is_active),
+        ) {
+            (_, true) => {
+                color = self.named_colors.background.0;
+            }
+            (true, false) => {
+                color = self.named_colors.foreground;
+            }
+            (false, false) => {}
         }
 
         let mut style = FragmentStyle {
             color,
-            background_color: Some(background_color),
+            background_color,
             font_attrs: font_attrs.into(),
             ..FragmentStyle::default()
         };
@@ -626,7 +615,7 @@ impl Renderer {
         style.decoration = decoration;
         style.decoration_color = decoration_color;
 
-        match self.cursor.state.content {
+        match cursor.state.content {
             CursorShape::Underline => {
                 style.decoration =
                     Some(FragmentStyleDecoration::Underline(UnderlineInfo {
@@ -646,26 +635,12 @@ impl Renderer {
             CursorShape::Hidden => {}
         }
 
-        (style, content)
-    }
-
-    #[inline]
-    pub fn set_ime(&mut self, ime_preedit: Option<&Preedit>) {
-        if let Some(preedit) = ime_preedit {
-            if let Some(content) = preedit.text.chars().next() {
-                self.cursor.content = content;
-                self.is_ime_enabled = true;
-                return;
-            }
+        if !is_active {
+            style.decoration = None;
+            style.cursor = Some(SugarCursor::HollowBlock(cursor_color));
         }
 
-        self.is_ime_enabled = false;
-        self.cursor.content = self.cursor.content_ref;
-    }
-
-    #[inline]
-    pub fn set_selection(&mut self, selection_range: Option<SelectionRange>) {
-        self.selection_range = selection_range;
+        (style, content)
     }
 
     #[inline]
@@ -673,61 +648,88 @@ impl Renderer {
         self.is_vi_mode_enabled = is_vi_mode_enabled;
     }
 
-    #[allow(clippy::too_many_arguments)]
+    #[inline]
     pub fn prepare_term(
         &mut self,
-        rows: &[Row<Square>],
-        cursor: CursorState,
         sugarloaf: &mut Sugarloaf,
-        context_manager: &crate::context::ContextManager<rio_backend::event::EventProxy>,
-        display_offset: i32,
-        has_blinking_enabled: bool,
+        context_manager: &mut ContextManager<EventProxy>,
         hints: &mut Option<HintMatches>,
         focused_match: &Option<RangeInclusive<Pos>>,
     ) {
-        let layout = sugarloaf.layout();
-        self.cursor.state = cursor;
-        let mut is_cursor_visible = self.cursor.state.is_visible();
-
-        self.font_size = layout.font_size;
-        self.term_has_blinking_enabled = has_blinking_enabled;
-
-        // Only blink cursor if does not contain selection
-        let has_selection = self.selection_range.is_some();
-        if !has_selection && self.has_blinking_enabled() {
-            let mut should_blink = true;
-            if let Some(last_typing_time) = self.last_typing {
-                if last_typing_time.elapsed() < Duration::from_secs(1) {
-                    should_blink = false;
-                }
-            }
-
-            if should_blink {
-                self.is_blinking = !self.is_blinking;
-                is_cursor_visible = self.is_blinking;
-            }
-        }
-
         let content = sugarloaf.content();
+        let grid = context_manager.current_grid_mut();
+        let active_index = grid.current;
 
-        // let start = std::time::Instant::now();
-        for (i, row) in rows.iter().enumerate() {
-            let has_cursor = is_cursor_visible && self.cursor.state.pos.row == i;
-            self.create_line(
-                content,
-                row,
-                has_cursor,
-                Line((i as i32) - display_offset),
-                hints,
-                focused_match,
-            );
+        for (index, grid_context) in grid.contexts_mut().iter_mut().enumerate() {
+            let is_active = active_index == index;
+            let context = grid_context.context_mut();
+            let rich_text_id = context.rich_text_id;
+            let renderable_content = context.renderable_content();
+            let mut is_cursor_visible = renderable_content.is_cursor_visible
+                && renderable_content.cursor.state.is_visible();
+            if !is_active && renderable_content.cursor.state.is_visible() {
+                is_cursor_visible = true;
+            }
+
+            let display_offset = renderable_content.display_offset;
+            let strategy = if is_active && hints.is_some() {
+                &RenderableContentStrategy::Full
+            } else {
+                &renderable_content.strategy
+            };
+
+            match strategy {
+                RenderableContentStrategy::Full => {
+                    content.sel(rich_text_id);
+                    content.clear();
+                    for (i, row) in renderable_content.inner.iter().enumerate() {
+                        let has_cursor = is_cursor_visible
+                            && renderable_content.cursor.state.pos.row == i;
+                        self.create_line(
+                            content,
+                            row,
+                            has_cursor,
+                            None,
+                            Line((i as i32) - display_offset),
+                            renderable_content,
+                            hints,
+                            focused_match,
+                            is_active,
+                        );
+                    }
+                    content.build();
+                }
+                RenderableContentStrategy::Lines(lines) => {
+                    content.sel(rich_text_id);
+                    for line in lines {
+                        let line = *line;
+                        if let Some(line_data) = renderable_content.inner.get(line) {
+                            let has_cursor = is_cursor_visible
+                                && renderable_content.cursor.state.pos.row == line;
+                            content.clear_line(line);
+                            self.create_line(
+                                content,
+                                line_data,
+                                has_cursor,
+                                Some(line),
+                                Line((line as i32) - display_offset),
+                                renderable_content,
+                                hints,
+                                focused_match,
+                                is_active,
+                            );
+                        }
+                    }
+                }
+                RenderableContentStrategy::Noop => {}
+            }
         }
-        // let duration = start.elapsed();
-        // println!("Total loop rows: {:?}", duration);
 
+        let window_size = sugarloaf.window_size();
+        let scale_factor = sugarloaf.scale_factor();
         let mut objects = Vec::with_capacity(30);
         self.navigation.build_objects(
-            (layout.width, layout.height, layout.dimensions.scale),
+            (window_size.width, window_size.height, scale_factor),
             &self.named_colors,
             context_manager,
             self.active_search.is_some(),
@@ -738,13 +740,17 @@ impl Renderer {
             search::draw_search_bar(
                 &mut objects,
                 &self.named_colors,
-                (layout.width, layout.height, layout.dimensions.scale),
+                (window_size.width, window_size.height, scale_factor),
                 active_search_content,
             );
 
             self.active_search = None;
         }
 
+        for rte in context_manager.grid_objects() {
+            objects.push(rte);
+        }
+
         sugarloaf.set_objects(objects);
     }
 }
