@@ -2,12 +2,14 @@ pub mod routes;
 mod window;
 use crate::event::EventProxy;
 use crate::router::window::{configure_window, create_window_builder};
+use crate::scheduler::{Scheduler, TimerId, Topic};
 use crate::screen::{Screen, ScreenWindowProperties};
 use assistant::Assistant;
 use raw_window_handle::{HasDisplayHandle, HasWindowHandle};
 use rio_backend::clipboard::Clipboard;
 use rio_backend::config::Config as RioConfig;
 use rio_backend::error::{RioError, RioErrorLevel, RioErrorType};
+use rio_backend::event::{EventPayload, RioEvent, RioEventType};
 use rio_window::event_loop::ActiveEventLoop;
 use rio_window::keyboard::{Key, NamedKey};
 #[cfg(not(any(target_os = "macos", windows)))]
@@ -19,7 +21,7 @@ use routes::{assistant, RoutePath};
 use rustc_hash::FxHashMap;
 use std::cell::RefCell;
 use std::rc::Rc;
-use std::time::Duration;
+use std::time::{Duration, Instant};
 
 // 𜱭𜱭 unicode is not available yet for all OS
 // https://www.unicode.org/charts/PDF/Unicode-16.0/U160-1CC00.pdf
@@ -56,13 +58,33 @@ impl Route<'_> {
         self.window.winit_window.request_redraw();
     }
 
+    pub fn request_frame(&mut self, scheduler: &mut Scheduler) {
+        let timer_id =
+            TimerId::new(Topic::RenderRoute, self.window.screen.ctx().current_route());
+        let event = EventPayload::new(
+            RioEventType::Rio(RioEvent::Render),
+            self.window.winit_window.id(),
+        );
+
+        if let Some(limit) = self.window.wait_until() {
+            self.window.start_render_timestamp();
+            scheduler.schedule(event, limit, false, timer_id);
+        } else {
+            self.window.start_render_timestamp();
+            self.request_redraw();
+        }
+    }
+
     #[inline]
     pub fn update_config(
         &mut self,
         config: &RioConfig,
         db: &rio_backend::sugarloaf::font::FontLibrary,
+        should_update_font: bool,
     ) {
-        self.window.screen.update_config(config, db);
+        self.window
+            .screen
+            .update_config(config, db, should_update_font);
     }
 
     #[inline]
@@ -250,6 +272,36 @@ impl Router<'_> {
         self.config_route = Some(id);
     }
 
+    pub fn open_config_split(&mut self, config: &RioConfig) {
+        let current_config: RioConfig = config.clone();
+        let editor = config.editor.clone();
+        let mut args = editor.args;
+        args.push(
+            rio_backend::config::config_file_path()
+                .display()
+                .to_string(),
+        );
+        let new_config = RioConfig {
+            shell: rio_backend::config::Shell {
+                program: editor.program,
+                args,
+            },
+            ..current_config
+        };
+
+        let window_id = match self.get_focused_route() {
+            Some(window_id) => window_id,
+            None => return,
+        };
+
+        let route = match self.routes.get_mut(&window_id) {
+            Some(window) => window,
+            None => return,
+        };
+
+        route.window.screen.split_right_with_config(new_config);
+    }
+
     #[inline]
     pub fn create_window<'a>(
         &'a mut self,
@@ -324,7 +376,9 @@ impl Router<'_> {
 pub struct RouteWindow<'a> {
     pub is_focused: bool,
     pub is_occluded: bool,
-    pub frame_time_limit: Option<Duration>,
+    has_fps_target: bool,
+    pub render_timestamp: Instant,
+    pub vblank_interval: Duration,
     pub winit_window: Window,
     pub screen: Screen<'a>,
     #[cfg(target_os = "macos")]
@@ -336,6 +390,41 @@ impl<'a> RouteWindow<'a> {
         configure_window(&self.winit_window, config);
     }
 
+    pub fn start_render_timestamp(&mut self) {
+        self.render_timestamp = Instant::now();
+    }
+
+    pub fn wait_until(&self) -> Option<Duration> {
+        let elapsed_time = Instant::now()
+            .duration_since(self.render_timestamp)
+            .as_millis() as u64;
+        let vblank_interval = self.vblank_interval.as_millis() as u64;
+
+        match vblank_interval >= elapsed_time {
+            true => Some(Duration::from_millis(vblank_interval - elapsed_time)),
+            // false => None,
+            false => Some(Duration::from_millis(vblank_interval.wrapping_sub(1))),
+        }
+    }
+
+    pub fn update_vblank_interval(&mut self) {
+        if !self.has_fps_target {
+            // Get the display vblank interval.
+            let monitor_vblank_interval = 1_000_000.
+                / self
+                    .winit_window
+                    .current_monitor()
+                    .and_then(|monitor| monitor.refresh_rate_millihertz())
+                    .unwrap_or(60_000) as f64;
+
+            // Now convert it to micro seconds.
+            let monitor_vblank_interval =
+                Duration::from_micros((1000. * monitor_vblank_interval) as u64);
+
+            self.vblank_interval = monitor_vblank_interval;
+        }
+    }
+
     #[allow(clippy::too_many_arguments)]
     pub fn from_target<'b>(
         event_loop: &'b ActiveEventLoop,
@@ -380,16 +469,36 @@ impl<'a> RouteWindow<'a> {
         )
         .expect("Screen not created");
 
-        let frame_time_limit = if config.renderer.max_fps == 0 {
-            None
-        } else {
-            Some(Duration::from_millis(
-                1000 / config.renderer.max_fps.clamp(1, 1000),
-            ))
-        };
+        #[cfg(target_os = "windows")]
+        {
+            // On windows cloak (hide) the window initially, we later reveal it after the first draw.
+            // This is a workaround to hide the "white flash" that occurs during application startup.
+            use rio_window::platform::windows::WindowExtWindows;
+            winit_window.set_cloaked(false);
+        }
+
+        // Get the display vblank interval.
+        let monitor_vblank_interval = 1_000_000.
+            / winit_window
+                .current_monitor()
+                .and_then(|monitor| monitor.refresh_rate_millihertz())
+                .unwrap_or(60_000) as f64;
+
+        // Now convert it to micro seconds.
+        let mut monitor_vblank_interval =
+            Duration::from_micros((1000. * monitor_vblank_interval) as u64);
+        let mut has_fps_target = false;
+
+        if let Some(target_fps) = config.renderer.target_fps {
+            monitor_vblank_interval =
+                Duration::from_millis(1000 / target_fps.clamp(1, 1000));
+            has_fps_target = true;
+        }
 
         Self {
-            frame_time_limit,
+            vblank_interval: monitor_vblank_interval,
+            has_fps_target,
+            render_timestamp: Instant::now(),
             is_focused: true,
             is_occluded: false,
             winit_window,
